@@ -1,6 +1,7 @@
 /*
-  DF-SHOW - A clone of 'SHOW' directory browser from DF-EDIT by Larry Kroeker
-  Copyright (C) 2018-2019  Robert Ian Hawdon
+  DF-SHOW: An interactive directory/file browser written for Unix-like systems.
+  Based on the applications from the PC-DOS DF-EDIT suite by Larry Kroeker.
+  Copyright (C) 2018-2024  Robert Ian Hawdon
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,11 +26,18 @@
 #include <libconfig.h>
 #include <dirent.h>
 #include <errno.h>
+#include <libintl.h>
+#include <locale.h>
+#include "menu.h"
+#include "display.h"
+#include "settings.h"
 #include "common.h"
 #include "colors.h"
+#include "input.h"
+#include "banned.h"
+#include "i18n.h"
 
 int lightColorPair[256];
-// int commandL, infoL, inputL, selectL, displayL, dangerL, dirL, slinkL, exeL, suidL, sgidL, hiliteL = 0;
 
 int colorThemePos = 0;
 int totalItemCount = 16;
@@ -44,7 +52,7 @@ colorPairs colors[256];
 
 char fgbgLabel[11];
 
-char errmessage[256];
+extern char *errmessage;
 
 extern int colormode;
 extern int c;
@@ -53,6 +61,8 @@ extern int * pc;
 extern char globalConfLocation[128];
 extern char homeConfLocation[128];
 
+extern int viewMode;
+
 menuDef *colorMenu;
 int colorMenuSize = 0;
 wchar_t *colorMenuLabel;
@@ -65,12 +75,12 @@ void processListThemes(const char * pathName)
   char currentFile[1024];
   config_t cfg;
   config_setting_t *group;
-  strcpy(currentPath, pathName);
+  memcpy(currentPath, pathName, (strlen(pathName) + 1));
   dfDir = opendir ( currentPath );
   if (access (currentPath, F_OK) != -1){
     if (dfDir){
       while ( ( res = readdir (dfDir))){
-        sprintf(currentFile, "%s/%s", currentPath, res->d_name);
+        snprintf(currentFile, 1024, "%s/%s", currentPath, res->d_name);
         if (!check_dir(currentFile)){
           config_init(&cfg);
           config_read_file(&cfg, currentFile);
@@ -160,58 +170,58 @@ int itemLookup(int menuPos){
 void setColorPairs(int pair, int foreground, int background, int bold){
   switch(pair){
   case 1:
-    strcpy(colors[pair].name, "command");
+    snprintf(colors[pair].name, 24, "command");
     break;
   case 2:
-    strcpy(colors[pair].name, "info");
+    snprintf(colors[pair].name, 24, "info");
     break;
   case 3:
-    strcpy(colors[pair].name, "input");
+    snprintf(colors[pair].name, 24, "input");
     break;
   case 4:
-    strcpy(colors[pair].name, "select");
+    snprintf(colors[pair].name, 24, "select");
     break;
   case 5:
-    strcpy(colors[pair].name, "display");
+    snprintf(colors[pair].name, 24, "display");
     break;
   case 6:
-    strcpy(colors[pair].name, "danger");
+    snprintf(colors[pair].name, 24, "danger");
     break;
   case 7:
-    strcpy(colors[pair].name, "dir");
+    snprintf(colors[pair].name, 24, "dir");
     break;
   case 8:
-    strcpy(colors[pair].name, "symlink");
+    snprintf(colors[pair].name, 24, "symlink");
     break;
   case 9:
-    strcpy(colors[pair].name, "exec");
+    snprintf(colors[pair].name, 24, "exec");
     break;
   case 10:
-    strcpy(colors[pair].name, "suid");
+    snprintf(colors[pair].name, 24, "suid");
     break;
   case 11:
-    strcpy(colors[pair].name, "sgid");
+    snprintf(colors[pair].name, 24, "sgid");
     break;
   case 12:
-    strcpy(colors[pair].name, "hilite");
+    snprintf(colors[pair].name, 24, "hilite");
     break;
   case 13:
-    strcpy(colors[pair].name, "error");
+    snprintf(colors[pair].name, 24, "error");
     break;
   case 14:
-    strcpy(colors[pair].name, "heading");
+    snprintf(colors[pair].name, 24, "heading");
     break;
   case 15:
-    strcpy(colors[pair].name, "deadlink");
+    snprintf(colors[pair].name, 24, "deadlink");
     break;
   case 16:
-    strcpy(colors[pair].name, "sticky");
+    snprintf(colors[pair].name, 24, "sticky");
     break;
   case 17:
-    strcpy(colors[pair].name, "sticky-ow");
+    snprintf(colors[pair].name, 24, "sticky-ow");
     break;
   default:
-    sprintf(colors[pair].name, "undef-%i", pair);
+    snprintf(colors[pair].name, 24, "undef-%i", pair);
     break;
   }
   if (COLORS < 9){
@@ -289,18 +299,19 @@ void saveTheme(){
   int e, f, i;
   char filename[1024];
   char * rewrite;
+  int curPos=0;
   move(0,0);
   clrtoeol();
-  printMenu(0,0, "Save Colors - Enter pathname:");
-  move(0,30);
-  rewrite = malloc(sizeof(char) * strlen(dirFromPath(homeConfLocation) +1));
-  sprintf(rewrite, "%s/", dirFromPath(homeConfLocation));
+  curPos = (printMenu(0,0, _("Save Colors - Enter pathname:")) + 1);
+  move(0,curPos);
+  rewrite = malloc(sizeof(char) * (strlen(dirFromPath(homeConfLocation)) +1));
+  snprintf(rewrite, (strlen(dirFromPath(homeConfLocation)) +1), "%s/", dirFromPath(homeConfLocation));
   e = readline(filename, 1024, rewrite);
   free(rewrite);
   if ( e == 0 ){
     if (check_first_char(filename, "~")){
       rewrite = str_replace(filename, "~", getenv("HOME"));
-      strcpy(filename, rewrite);
+      memcpy(filename, rewrite, (strlen(rewrite) + 1));
       free(rewrite);
     }
     config_init(&cfg);
@@ -319,7 +330,7 @@ void saveTheme(){
     if (access(dirFromPath(filename), W_OK) == 0){
       if (check_file(filename)){
         curs_set(FALSE);
-        printMenu(0,0, "File exists. Replace? (!Yes/!No)");
+        printMenu(0,0, _("File exists. Replace? (!Yes/!No)"));
         *pc = getch10th();
         if (*pc == 'y'){
           config_write_file(&cfg, filename);
@@ -346,12 +357,14 @@ void saveTheme(){
           createParentDirs(filename);
           goto saveTheme;
         } else {
-          sprintf(errmessage, "Error: %s", strerror(errno));
+          setDynamicChar(&errmessage, _("Error: %s"), strerror(errno));
           topLineMessage(errmessage);
+          free(errmessage);
         }
       } else {
-        sprintf(errmessage, "Error: %s", strerror(errno));
+        setDynamicChar(&errmessage, _("Error: %s"), strerror(errno));
         topLineMessage(errmessage);
+        free(errmessage);
       }
     }
     config_destroy(&cfg);
@@ -361,7 +374,7 @@ void saveTheme(){
 
 int applyTheme(const char *filename){
   config_t cfg;
-  config_setting_t *root, *setting, *group, *array;
+  config_setting_t *setting, *group, *array;
   int groupLen, i, h;
   setenv("DFS_THEME", objectFromPath(filename), 1);
   config_init(&cfg);
@@ -384,7 +397,7 @@ int applyTheme(const char *filename){
     refreshColors();
     return(0);
   } else {
-    topLineMessage("Error parsing theme file.");
+    topLineMessage(_("Error parsing theme file."));
     return(1);
   }
   return(1);
@@ -394,18 +407,19 @@ void loadTheme(){
   int e;
   char filename[1024];
   char * rewrite;
+  int curPos = 0;
   move(0,0);
   clrtoeol();
-  printMenu(0,0, "Load Colors - Enter pathname:");
-  move(0,30);
-  rewrite = malloc(sizeof(char) * strlen(dirFromPath(homeConfLocation) +1));
-  sprintf(rewrite, "%s/", dirFromPath(homeConfLocation));
+  curPos = (printMenu(0,0, _("Load Colors - Enter pathname:")) + 1 );
+  move(0,curPos);
+  rewrite = malloc(sizeof(char) * (strlen(dirFromPath(homeConfLocation)) + 1));
+  snprintf(rewrite, (strlen(dirFromPath(homeConfLocation)) + 1), "%s/", dirFromPath(homeConfLocation));
   e = readline(filename, 1024, rewrite);
   free(rewrite);
   if ( e == 0 ){
     if (check_first_char(filename, "~")){
       rewrite = str_replace(filename, "~", getenv("HOME"));
-      strcpy(filename, rewrite);
+      memcpy(filename, rewrite, (strlen(rewrite) + 1));
       free(rewrite);
     }
     if (check_file(filename) ){
@@ -414,7 +428,7 @@ void loadTheme(){
       applyTheme(filename);
     } else {
       curs_set(FALSE);
-      topLineMessage("Error: Unable to read file");
+      topLineMessage(_("Error: Unable to read file"));
       // curs_set(TRUE);
     }
   }
@@ -427,13 +441,13 @@ void loadAppTheme(const char *themeName)
   // Ignore if the theme requested is called 'default'
   if (strcmp(themeName, "default") && strcmp(themeName, "\0")){
       rewrite = malloc(sizeof(char) * (strlen(dirFromPath(homeConfLocation)) + strlen(themeName) + 2));
-      sprintf(rewrite, "%s/%s", dirFromPath(homeConfLocation), themeName);
+      snprintf(rewrite, (strlen(dirFromPath(homeConfLocation)) + strlen(themeName) + 2), "%s/%s", dirFromPath(homeConfLocation), themeName);
       if (check_file(rewrite)){
         applyTheme(rewrite);
       } else {
         free(rewrite);
         rewrite = malloc(sizeof(char) * (strlen(DATADIR) + strlen(themeName) + 2));
-        sprintf(rewrite, "%s/%s", DATADIR, themeName);
+        snprintf(rewrite, (strlen(DATADIR) + strlen(themeName) + 2), "%s/%s", DATADIR, themeName);
         if (check_file(rewrite)){
           applyTheme(rewrite);
         }
@@ -650,28 +664,28 @@ void theme_menu_inputs()
         //
         if (themeModified == 1){
           curs_set(FALSE);
-          topLineMessage("Save theme before using as default.");
+          topLineMessage(_("Save theme before using as default."));
         } else {
           if (access(dirFromPath(homeConfLocation), W_OK) != 0) {
             createParentDirs(homeConfLocation);
           }
           if (useTheme(homeConfLocation)){
-            sprintf(useThemeMessage, "Default theme has been set to [%s].", getenv("DFS_THEME"));
+            snprintf(useThemeMessage, 256, _("Default theme has been set to [%s]."), getenv("DFS_THEME"));
             topLineMessage(useThemeMessage);
           } else {
-            topLineMessage("An error occurred setting the default theme.");
+            topLineMessage(_("An error occurred setting the default theme."));
           }
         }
         themeBuilder();
       } else if (*pc == 258 || *pc ==10){
         if (colorThemePos < totalItemCount){
           colorThemePos++;
-          themeBuilder();
+          setCursorPos(colorThemePos - 1);
         }
       } else if (*pc == 259){
         if (colorThemePos > 0) {
           colorThemePos--;
-          themeBuilder();
+          setCursorPos(colorThemePos + 1);
         }
       } else if (*pc == 260 || *pc == 261){
         // Do Nothing
@@ -737,8 +751,6 @@ void setDefaultTheme(){
 
 void setColors(int pair)
 {
-  // endwin();
-  // printf("CP: %i\n", pair);
   attron(COLOR_PAIR(pair));
   if (lightColorPair[pair]){
     attron(A_BOLD);
@@ -747,96 +759,104 @@ void setColors(int pair)
   }
 }
 
+void setCursorPos(int prev)
+{
+  setColors(DISPLAY_PAIR);
+  if (prev >= 0){
+    // attroff(A_REVERSE);
+    mvprintw(prev + 2, 1, " ");
+  }
+  attron(A_REVERSE);
+  mvprintw(colorThemePos + 2, 1, " ");
+  attroff(A_REVERSE);
+}
+
 void themeBuilder()
 {
+  viewMode = 2;
   clear();
   if (bgToggle){
-    strcpy(fgbgLabel, "background");
+    snprintf(fgbgLabel, 11, _("background"));
   } else {
-    strcpy(fgbgLabel, "foreground");
+    snprintf(fgbgLabel, 11, _("foreground"));
   }
   wPrintMenu(0,0,colorMenuLabel);
 
   setColors(COMMAND_PAIR);
-  mvprintw(2, 4, "Command lines");
+  mvprintw(2, 4, _("Command lines"));
   setColors(DISPLAY_PAIR);
-  mvprintw(3, 4, "Display lines");
+  mvprintw(3, 4, _("Display lines"));
   setColors(ERROR_PAIR);
-  mvprintw(4, 4, "Error messages");
+  mvprintw(4, 4, _("Error messages"));
   setColors(INFO_PAIR);
-  mvprintw(5, 4, "Information lines");
+  mvprintw(5, 4, _("Information lines"));
   setColors(HEADING_PAIR);
-  mvprintw(6, 4, "Heading lines");
+  mvprintw(6, 4, _("Heading lines"));
   setColors(DANGER_PAIR);
-  mvprintw(7, 4, "Danger lines");
+  mvprintw(7, 4, _("Danger lines"));
   setColors(SELECT_PAIR);
-  mvprintw(8, 4, "Selected block lines");
+  mvprintw(8, 4, _("Selected block lines"));
   setColors(HILITE_PAIR);
-  mvprintw(9, 4, "Highlight");
+  mvprintw(9, 4, _("Highlight"));
   setColors(INPUT_PAIR);
-  mvprintw(10, 4, "Text input");
+  mvprintw(10, 4, _("Text input"));
   setColors(DIR_PAIR);
-  mvprintw(11, 4, "Directories");
+  mvprintw(11, 4, _("Directories"));
   setColors(SLINK_PAIR);
-  mvprintw(12, 4, "Symbolic links");
+  mvprintw(12, 4, _("Symbolic links"));
   setColors(DEADLINK_PAIR);
-  mvprintw(13, 4, "Orphened symbolic links");
+  mvprintw(13, 4, _("Orphened symbolic links"));
   setColors(EXE_PAIR);
-  mvprintw(14, 4, "Executable files");
+  mvprintw(14, 4, _("Executable files"));
   setColors(SUID_PAIR);
-  mvprintw(15, 4, "Set user identification");
+  mvprintw(15, 4, _("Set user identification"));
   setColors(SGID_PAIR);
-  mvprintw(16, 4, "Set group identification");
+  mvprintw(16, 4, _("Set group identification"));
   setColors(STICKY_PAIR);
-  mvprintw(17, 4, "Sticky bit directory");
+  mvprintw(17, 4, _("Sticky bit directory"));
   setColors(STICKY_OW_PAIR);
-  mvprintw(18, 4, "Sticky bit directory - other writable");
+  mvprintw(18, 4, _("Sticky bit directory - other writable"));
 
   setColors(DEFAULT_COLOR_PAIR);
-  mvprintw(2, 45, "!-Default      ");
+  mvprintw(2, 45, _("!-Default      "));
   setColors(DEFAULT_BOLD_PAIR);
-  mvprintw(3, 45, "?-Default Bold ");
+  mvprintw(3, 45, _("?-Default Bold "));
   setColors(COLORMENU_PAIR_0);
-  mvprintw(4, 45, "0-Black        ");
+  mvprintw(4, 45, _("0-Black        "));
   setColors(COLORMENU_PAIR_1);
-  mvprintw(5, 45, "1-Red          ");
+  mvprintw(5, 45, _("1-Red          "));
   setColors(COLORMENU_PAIR_2);
-  mvprintw(6, 45, "2-Green        ");
+  mvprintw(6, 45, _("2-Green        "));
   setColors(COLORMENU_PAIR_3);
-  mvprintw(7, 45, "3-Brown        ");
+  mvprintw(7, 45, _("3-Brown        "));
   setColors(COLORMENU_PAIR_4);
-  mvprintw(8, 45, "4-Blue         ");
+  mvprintw(8, 45, _("4-Blue         "));
   setColors(COLORMENU_PAIR_5);
-  mvprintw(9, 45, "5-Magenta      ");
+  mvprintw(9, 45, _("5-Magenta      "));
   setColors(COLORMENU_PAIR_6);
-  mvprintw(10, 45, "6-Cyan         ");
+  mvprintw(10, 45, _("6-Cyan         "));
   setColors(COLORMENU_PAIR_7);
-  mvprintw(11, 45, "7-Light Gray   ");
+  mvprintw(11, 45, _("7-Light Gray   "));
   setColors(COLORMENU_PAIR_8);
-  mvprintw(12, 45, "8-Dark Gray    ");
+  mvprintw(12, 45, _("8-Dark Gray    "));
   setColors(COLORMENU_PAIR_9);
-  mvprintw(13, 45, "9-Light Red    ");
+  mvprintw(13, 45, _("9-Light Red    "));
   setColors(COLORMENU_PAIR_A);
-  mvprintw(14, 45, "A-Light Green  ");
+  mvprintw(14, 45, _("A-Light Green  "));
   setColors(COLORMENU_PAIR_B);
-  mvprintw(15, 45, "B-Yellow       ");
+  mvprintw(15, 45, _("B-Yellow       "));
   setColors(COLORMENU_PAIR_C);
-  mvprintw(16, 45, "C-Light Blue   ");
+  mvprintw(16, 45, _("C-Light Blue   "));
   setColors(COLORMENU_PAIR_D);
-  mvprintw(17, 45, "D-Light Magenta");
+  mvprintw(17, 45, _("D-Light Magenta"));
   setColors(COLORMENU_PAIR_E);
-  mvprintw(18, 45, "E-Light Cyan   ");
+  mvprintw(18, 45, _("E-Light Cyan   "));
   setColors(COLORMENU_PAIR_F);
-  mvprintw(19, 45, "F-White        ");
+  mvprintw(19, 45, _("F-White        "));
 
   setColors(DEFAULT_BOLD_PAIR);
-  mvprintw(22, 22, "Select 0 to F for desired %s color", fgbgLabel);
-
-  // curs_set(TRUE);
-  // move(colorThemePos + 2, 1);
-  attron(A_REVERSE);
-  mvprintw(colorThemePos + 2, 1, " ");
-  attroff(A_REVERSE);
+  mvprintw(22, 22, _("Select 0 to F for desired %s color"), fgbgLabel);
 
+  setCursorPos(-1);
 
 }
