@@ -1,6 +1,6 @@
-void generateDefaultMenus();
-void refreshMenuLabels();
-void unloadMenuLabels();
+void generateDefaultShowMenus();
+void refreshShowMenuLabels();
+void unloadShowMenuLabels();
 int touchType();
 time_t touchTimeInput(int type);
 int symLinkLocation();
@@ -10,6 +10,8 @@ void modify_key_menu_inputs();
 void modify_owner_input();
 void modify_group_input();
 void modify_permissions_input();
+void modify_context_menu_inputs();
+void modify_context_input(int mode);
 void show_directory_input();
 void directory_view_menu_inputs();
 void global_menu_inputs();
@@ -28,5 +30,4 @@ void copy_multi_file(results* ob, char *dest);
 void rename_multi_file_input(results* ob, char *input);
 int huntCaseSelectInput();
 void huntInput(int selected, int charcase);
-void refreshDirectory(char *sortmode, int origtopfileref, int origselected, int destructive);
-int sanitizeTopFileRef(int topfileref);
+void refreshDirectory(char *sortmode, int origlineStart, int origselected, int destructive);
